import * as PImage from 'pureimage';
import { createWriteStream } from 'fs';
import { tmpdir } from 'os';
import { join } from 'path';
import { v4 as uuid } from 'uuid';

export class TestImage {
	public static async Frame (width: number, height: number, edge : number = 2) : Promise<string> {
		const id : string = uuid();
		const filePath : string = join(tmpdir(), `frame-${id}.png`);
		const img : PImage.Bitmap = PImage.make(width, height);
		const ctx : PImage.Context = img.getContext('2d');
		ctx.fillStyle = 'black';
		ctx.fillRect(0, 0, width, height);
		ctx.fillStyle = 'red';
		ctx.fillRect(0, 0, width, edge);
		ctx.fillRect(0, 0, edge, height);
		ctx.fillRect(0, height - edge, width, edge);
		ctx.fillRect(width - edge, 0, edge, height);
		try {
			await PImage.encodePNGToStream(img, createWriteStream(filePath));
		} catch (err) {
			throw err;
		}
		return filePath;
	}

	public static async Focus (width : number, height : number, count : number = 20) : Promise<string> {
		const id : string = uuid();
		const filePath : string = join(tmpdir(), `frame-${id}.png`);
		const img : PImage.Bitmap = PImage.make(width, height);
		const ctx : PImage.Context = img.getContext('2d');

		const half : number = Math.round(count / 2)
		const dpr : number = 1;
		const w : number = width / dpr;
		const h : number = height / dpr;
		const longest : number = w >= h ? w * 1.5 : h * 1.5;
		const opp : number = Math.tan(360 / (count * 32)) * longest / 10;
		ctx.fillStyle = 'black';
		ctx.fillRect(0, 0, w, h);
		ctx.fillStyle = 'white';
		for (let i = 0; i < count; i++) {
			ctx.beginPath()
			ctx.moveTo(w / 2, h / 2)
			ctx.lineTo((w / 2) + opp, longest)
			ctx.lineTo((w / 2) - opp, longest)
			ctx.fill()
			ctx.translate(w / 2, h / 2);
			ctx.rotate((360 / count) * Math.PI / 180)
			ctx.translate(- w / 2, -h / 2)
		}

		try {
			await PImage.encodePNGToStream(img, createWriteStream(filePath));
		} catch (err) {
			throw err;
		}
		return filePath;
	}
}

module.exports = { TestImage };